//
//  main.swift
//  PMUI_Mac_EventListener
//
//  Copyright © 2016 pmui. All rights reserved.
//

import Cocoa
import Foundation
import AppKit

class UserEventListener: NSObject, NSApplicationDelegate {
    
    
    var user_state = ""
    var timer  = NSTimer()
    var cdatetime = ""
    var active_countdown = 0;
    var checkFor10Mins = false
    
    override init() {
        
        super.init();
        self.user_state = "USER_IDLE"
        self.active_countdown = 0
        self.checkFor10Mins = true
        self.setTimeVariables(1)
        self.printUserEventJson(6)
        
        self.timer = NSTimer(timeInterval: 1.0, target: self, selector: #selector(UserEventListener.checkState), userInfo: nil, repeats: true)
        NSRunLoop.currentRunLoop().addTimer(timer, forMode: NSRunLoopCommonModes)
        
       
        startGlobalInputListeners()
        
        NSWorkspace.sharedWorkspace().notificationCenter.addObserver(self, selector: #selector(UserEventListener.sleepListener(_:)), name: NSWorkspaceWillSleepNotification, object: nil)
        NSWorkspace.sharedWorkspace().notificationCenter.addObserver(self, selector: #selector(UserEventListener.wakeUpListener(_:)) , name: NSWorkspaceDidWakeNotification, object: nil)
       

    }//end init

    
    func sleepListener(aNotification : NSNotification) {
      //On Sleep Event
        setTimeVariables(-1)
        print("{\"event\": {\"winEventId\":\"2222\",\"timestamp\":\"\(self.cdatetime)\",\"name\":\"COMPUTER_SLEEP\"}}");
        setbuf(__stdoutp, nil);

    }
    
    func wakeUpListener(aNotification : NSNotification) {
        setTimeVariables(-1)
        print("{\"event\": {\"winEventId\":\"1111\",\"timestamp\":\"\(self.cdatetime)\",\"name\":\"COMPUTER_AWAKE\"}}");
        setbuf(__stdoutp, nil);
       //on Wake Event -- run from idle and countdown 0
        self.user_state = "USER_IDLE"
        self.active_countdown = 0
        self.checkFor10Mins = true
        self.setTimeVariables(1)
        self.printUserEventJson(6)
        self.resetTimer()
    }
    //if countdown reaches 600 seconds (10 mins). change to USER_IDLE
    func checkState() {
         self.active_countdown = self.active_countdown + 1
        if(self.user_state=="USER_ACTIVE"){
            //self.active_countdown = self.active_countdown + 1
            self.checkFor10Mins = false
        }
        if(self.active_countdown >= 60 && self.user_state == "USER_ACTIVE"){
            user_state = "USER_IDLE"
            self.setTimeVariables(1)
            self.printUserEventJson(15)
            self.checkFor10Mins = true

        }
        if(self.active_countdown >= 600 && self.checkFor10Mins == true){
            user_state = "USER_IDLE"
            self.setTimeVariables(1)
            self.printUserEventJson(8)
            self.checkFor10Mins = false

        }
        
        
    }

    func setTimeVariables(padding:Int16){
        let date = NSDate()
        let calendar = NSCalendar.currentCalendar()
        let components = calendar.components([ .Year, .Month, .Day, .Hour, .Minute, .Second], fromDate: date)
        self.cdatetime = "\(components.month)-\(components.day)-\(components.year) \(components.hour):\(components.minute):\(components.second+padding )"
    }
    
    func startGlobalInputListeners(){
    
    NSEvent.addGlobalMonitorForEventsMatchingMask(
        [NSEventMask.KeyDownMask,  NSEventMask.LeftMouseDownMask, NSEventMask.RightMouseDownMask, NSEventMask.OtherMouseDownMask, NSEventMask.MouseMovedMask , NSEventMask.ScrollWheelMask], handler: {(event: NSEvent) in
           // print(event)
  
            self.setTimeVariables(2)
            if(self.user_state == "USER_IDLE"){
                
                self.user_state = "USER_ACTIVE";
                self.printUserEventJson(7)
            }
            
            self.resetTimer()
           
    })

    
    }
    
    func resetTimer(){

        self.active_countdown = 0
        self.timer.invalidate()
        self.timer = NSTimer(timeInterval: 1.0, target: self, selector: #selector(UserEventListener.checkState), userInfo: nil, repeats: true)
        NSRunLoop.currentRunLoop().addTimer(self.timer, forMode: NSRunLoopCommonModes)
    }
    
    func printUserEventJson(id: Int16){
        if(self.user_state == "USER_IDLE"){
            setTimeVariables(1)
        }
           print("{\"event\": {\"winEventId\":\"\(id)\",\"timestamp\":\"\(self.cdatetime)\",\"name\":\"\(self.user_state)\"}}");
        setbuf(__stdoutp, nil);

    }
    
    
    /*get privileges for application*/
    func applicationDidFinishLaunching(notification: NSNotification) {
        acquirePrivileges()
    }
    
    func acquirePrivileges() -> Bool {
        let accessEnabled = AXIsProcessTrustedWithOptions(
            [kAXTrustedCheckOptionPrompt.takeUnretainedValue() as String: true])
        
        if accessEnabled != true {
           // print("You PMUI EventListener in the System Preferences")
        }
        return accessEnabled == true
    }


} //end UserEventListener




// preparing main loop
let application = NSApplication.sharedApplication()
let applicationDelegate = UserEventListener()
application.delegate = applicationDelegate
application.activateIgnoringOtherApps(true)
application.run()



